/****************************************************************/
/*   IMPLEMENTATION OF THE CCITT WIDEBAND CODER WITH THE	*/
/*   WE(R) DSP16 DIGITAL SIGNAL PROCESSOR			*/
/*   August 1988, Doc# AP88-05DMOS				*/
/****************************************************************/
/*	 Copyright (c) 1988 AT&T -- All Rights Reserved		*/
/*   WE is a registered trademark of AT&T. AT&T reserves the	*/
/*   right to make changes to  the software described herein	*/
/*   without notice.  No liability is assumed as a result of	*/
/*   their  use or  application.  No rights under any patent	*/
/*   accompany the sale of product.				*/
/****************************************************************/
/* This program is coded according to				*/
/* the CCITT standard algorithm described in G.722.		*/
/* All the notations used here are based on the CCITT documents */
/* for ease in understanding the program.			*/
/****************************************************************/

/****************************************************************/
/*								*/
/*		Program Initialization				*/
/*								*/
/****************************************************************/

reset:	auc=0x0		/** RESET SECTION			**/
	y=0		/* Initialize quantizer scale factor	*/
	r0=NBL		/* adaptation in lower sub-band.	*/
	*r0=y
	r1=AL1		/* Initialize adaptive predictor and	*/
	r0=RLT1		/* reconstructed signal calculator	*/
	do 8 {		/* in lower sub-band.			*/
	*r0++=y
	*r1++=y
	}
	r0=PLT1
	*r0++=y
	*r0=y
	r0=NBH		/* Initialize quantizer scale factor	*/
	*r0=y		/* adaptation in higher sub-band.	*/
	r1=AH1		/* Initialize adaptive predictor and	*/
	r0=RH1		/* reconstructed signal calculator	*/
	do 8 {		/* in higher sub-band.			*/
	*r0++=y
	*r1++=y
	}
	r0=PH1
	*r0++=y
	*r0=y
	y=32		/* Initialize difference signal comput. */
	r0=DETL		/* and quantization in lower sub-band.	*/
	*r0=y
	y=8
	r0=DETH		/* Initialize difference signal comput. */
	*r0=y		/* and quantization in higher sub-band. */
	y=32640		/* Set the constant value for the	*/
	r0=temp1	/* operation.				*/
	*r0=y
	r0=temp2
	*r0=y

/****************************************************************/
/*								*/
/*			Main Program				*/
/*								*/
/****************************************************************/

/****************************************************************/
/*								*/
/* In the following program, it is assumed the input values are	*/
/* stored in XL and XH locations.				*/
/* These two values have the bit formant ( SSbbbbbbbbbbbbbb )	*/
/* S : sign indicator, b : binary.				*/
/* The output values of IH and IL are combined into the word	*/
/* format as							*/
/*	(IH1)(IH2)(IL1)(IL2)(IL3)(IL4)(IL5)(IL6)00000000	*/
/* stored in the OUT location.					*/
/*								*/
/****************************************************************/

/****************************************************************/
/*								*/
/*			Block 4L				*/
/*								*/
/****************************************************************/

b4al:	r1=DLT6		/**	FILTEZ			**/
	r2=SZL		/* Set initial points		*/
	r0=BL6
	y=0
	a1=y  x=*r0--	/* Clear a1, x = BL6		*/
	y=*r1--		/* y = DLT6			*/
	do 6 {
	a0=y  *r2=a1
	a0=a0+y		/* WDn = DLTn + DLTn		*/
	y=a0		/* y = WDn			*/
	p=x*y  a1=*r2	/* p = BLn * WDn		*/
	a0=p  x=*r0--
	a0=a0<<1	/* Rescale the result.		*/
	y=a0
	a1=a1+y  y=*r1--  /* Accumulate the result.	*/
	}
	*r2++=a1	/* a1 = SZL			*/
	a1=a1<<16
	redo 2		/**		FILTEP		**/
	*r2=a1		/* a1 = SPL			*/
	a1=*r2--	/* a1 = SPL			*/
	y=*r2++		/* y = SZL			*/
	a1=a1+y		/**		PREDIC		**/
	*r2=a1		/* r2=SL			*/

/****************************************************************/
/*								*/
/*			Block 1L				*/
/*								*/
/****************************************************************/

b1l:	r0=XL		/**		SUBTRA		**/
	y=*r2		/* y = SL			*/
	a1=*r0--	/* a1 = XL			*/
	a1=a1-y
	*r0=a1		/* a1=EL			*/
	a1=*r0
	y=0xffff
	a1=a1^y  y=*r0	/* a1 = 32767 - EL		*/
	if mi a1=y	/* a1 = WD			*/
	r0=MEL		/**		QUANTL		**/
	*r0=a1		/* Store WD in MEL.		*/
	r1=DETL
	x=6288*2
	y=*r1		/* y = DETL			*/
	p=x*y		/* p = (786 << 3) * DETL	*/
	a0=p		/* The following are the binary	*/
	y=a0		/* searching implementation.	*/
	a1-y y=*r1	/* CHECK ENTRY IS ABOVE 16?	*/
	if pl goto qua1
	x=2584*2	/* NO, 1 <= r2 <= 16		*/
	p=x*y
	a0=p
	y=a0
	a1-y y=*r1	/* CHECK ENTRY IS ABOVE 8?	*/
	if pl goto qua2
	x=1200*2	/* NO, 1 <= r2 <= 8		*/
	p=x*y
	a0=p
	y=a0
	a1-y y=*r1	/* CHECK ENTRY IS ABOVE 4?	*/
	if pl goto qua3
	x=576*2		/* NO, 1 <= r2 <= 4		*/
	p=x*y
	a0=p
	y=a0
	a1-y y=*r1	/* CHECK ENTRY IS ABOVE 2?	*/
	if pl goto qua4
	x=280*2		/* NO, 1 <= r2 <= 2		*/
	p=x*y
	a0=p
	y=a0
	a1-y
	r2=1
	a0=r2
	y=2
	if pl a0=y	/* SELECT a0 = 1 OR 2		*/
	goto finl
qua4:	x=880*2		/* YES, 3 <= r2 <= 4		*/
	p=x*y
	a0=p
	y=a0
	a1-y
	r2=3
	a0=r2
	y=4
	if pl a0=y	/* SELECT a0 = 3 OR 4		*/
	goto finl
qua3:	x=1864*2	/* YES 5 <= r2 <= 8		*/
	p=x*y
	a0=p
	y=a0
	a1-y y=*r1	/* CHECK ENTRY IS ABOVE 6?	*/
	if pl goto qua5
	x=1520*2	/* NO, 5 <= r2 <= 6		*/
	p=x*y
	a0=p
	y=a0
	a1-y
	r2=5
	a0=r2
	y=6
	if pl a0=y	/* SELECT a0 = 5 OR 6		*/
	goto finl
qua5:	x=2208*2	/* YES, 7 <= r2 <= 8		*/
	p=x*y
	a0=p
	y=a0
	a1-y
	r2=7
	a0=r2
	y=8
	if pl a0=y	/* SELECT a0 = 7 OR 8		*/
	goto finl
qua2:	x=4240*2	/* YES, 9 <= r2 <= 16		*/
	p=x*y
	a0=p
	y=a0
	a1-y y=*r1	/* CHECK ENTRY IS ABOVE 12?	*/
	if pl goto qua6
	x=3376*2	/* NO, 9 <= r2 <= 12		*/
	p=x*y
	a0=p
	y=a0
	a1-y y=*r1	/* CHECK ENTRY IS ABOVE 10?	*/
	if pl goto qua7
	x=2960*2	/* NO, 9 <= r2 <= 10		*/
	p=x*y
	a0=p
	y=a0
	a1-y
	r2=9
	a0=r2
	y=10
	if pl a0=y	/* SELECT a0 = 9 OR 10		*/
	goto finl
qua7:	x=3784*2	/* YES, 11 <= r2 <= 12		*/
	p=x*y
	a0=p
	y=a0
	a1-y
	r2=11
	a0=r2
	y=12
	if pl a0=y	/* SELECT a0 = 11 OR 12		*/
	goto finl
qua6:	x=5200*2	/* YES, 13 <= r2 <= 16		*/
	p=x*y
	a0=p
	y=a0
	a1-y y=*r1	/* CHECK ENTRY IS ABOVE 14?	*/
	if pl goto qua8
	x=4696*2	/* NO, 13 <= r2 <= 14		*/
	p=x*y
	a0=p
	y=a0
	a1-y
	r2=13
	a0=r2
	y=14
	if pl a0=y	/* SELECT a0 = 13 OR 14		*/
	goto finl
qua8:	x=5712*2	/* YES, 15 <= r2 <= 16		*/
	p=x*y
	a0=p
	y=a0
	a1-y
	r2=15
	a0=r2
	y=16
	if pl a0=y	/* SELECT a0 = 15 OR 16		*/
	goto finl
qua1:	x=12896*2	/* YES, 17 <= r2 <= 30		*/
	p=x*y
	a0=p
	y=a0
	a1-y y=*r1	/* CHECK ENTRY IS ABOVE 24?	*/
	if pl goto qua9
	x=8968*2	/* NO, 17 <= r2 <= 24		*/
	p=x*y
	a0=p
	y=a0
	a1-y y=*r1	/* CHECK ENTRY IS ABOVE 20?	*/
	if pl goto qua10
	x=7520*2	/* NO, 17 <= r2 <= 20		*/
	p=x*y
	a0=p
	y=a0
	a1-y y=*r1	/* CHECK ENTRY IS ABOVE 18?	*/
	if pl goto qua11
	x=6864*2	/* NO, 17 <= r2 <= 18		*/
	p=x*y
	a0=p
	y=a0
	a1-y
	r2=17
	a0=r2
	y=18
	if pl a0=y	/* SELECT a0 = 17 OR 18		*/
	goto finl
qua11:	x=8184*2	/* YES, 19 <= r2 <= 20		*/
	p=x*y
	a0=p
	y=a0
	a1-y
	r2=19
	a0=r2
	y=20
	if pl a0=y	/* SELECT a0 = 19 OR 20		*/
	goto finl
qua10:	x=10712*2	/* YES, 21 <= r2 <= 24		*/
	p=x*y
	a0=p
	y=a0
	a1-y y=*r1	/* CHECK ENTRY IS ABOVE 22?	*/
	if pl goto qua12
	x=9752*2	/* NO, 21 <= r2 <= 22		*/
	p=x*y
	a0=p
	y=a0
	a1-y
	r2=21
	a0=r2
	y=22
	if pl a0=y	/* SELECT a0 = 21 OR 22		*/
	goto finl
qua12:	x=11664*2	/* YES, 23 <= r2 <= 24		*/
	p=x*y
	a0=p
	y=a0
	a1-y
	r2=23
	a0=r2
	y=24
	if pl a0=y	/* SELECT a0 = 23 OR 24		*/
	goto finl
qua9:	x=20456		/* YES, 25 <= r2 <= 30		*/
	p=x*y
	a0=p
	a0=a0<<1
	y=a0
	a1-y y=*r1	/* CHECK ENTRY IS ABOVE 28?	*/
	if pl goto qua13
	x=15840*2	/* NO, 25 <= r2 <= 28		*/
	p=x*y
	a0=p
	y=a0
	a1-y y=*r1	/* CHECK ENTRY IS ABOVE 26?	*/
	if pl goto qua14
	x=14120*2	/* NO, 25 <= r2 <= 26		*/
	p=x*y
	a0=p
	y=a0
	a1-y
	r2=25
	a0=r2
	y=26
	if pl a0=y	/* SELECT a0 = 25 OR 26		*/
	goto finl
qua14:	x=17560		/* YES, 27 <= r2 <= 28		*/
	p=x*y
	a0=p
	a0=a0<<1
	y=a0
	a1-y
	r2=27
	a0=r2
	y=28
	if pl a0=y	/* SELECT a0 = 27 OR 28		*/
	goto finl
qua13:	x=23352		/* YES, 29 <= r2 <= 30		*/
	p=x*y
	a0=p
	a0=a0<<1
	y=a0
	a1-y
	r2=29
	a0=r2
	y=30
	if pl a0=y	/* SELECT a0 = 29 OR 30		*/
finl:	r2=a0
	y=PTIL		/* Point to the base of positive */
	r0=tem2		/* IL table.			 */
	a0=y  a1=*r0
	y=NTIL		/* Point to the base of negative */
	a1=a1		/* IL table.			 */
	if mi a0=y
	a1=r2
	y=a0		/* y = offset of the table.	*/
	a1=a1+y
	pt=a1		/* pt is the point of IL table.	*/
	y=*r1  x=*pt++	/* x = IL			*/
	r0=IL
	*r0=x		/* Store it in IL location.	*/

/****************************************************************/
/*								*/
/*			Block 2L				*/
/*								*/
/****************************************************************/

b2l:	a0=x		/**		INVQAL			**/
	a0=a0>>1
	a0=a0>>1	/* a0 = RIL = IL >>> 2			*/
	y=TIL4		/* Point to the base of IL4 table.	*/
	a0=a0+y
	pt=a0
	y=*r0  x=*pt++	/* x = IL4				*/
	a0=x
	a1=a0
	if mi a0=-a0
	j=a0		/* j=IL4				*/
	y=TQQ4		/* Point to the base of QQ4 table.	*/
	a0=a0+y
	pt=a0
	y=*r1  x=*pt++	/* r1=DETL, x = QQ4(IL4)		*/
	a0=x
	a0=a0>>1
	a0=a0<<4	/* a0 = WD1 = QQ4(IL4) << 3		*/
	a1=a1
	if mi a0=-a0
	x=a0		/* x = WD2				*/
	p=x*y		/* p = DLT = DETL * WD2			*/
	a0=p
	a0=a0<<1
	r0=DLT
	*r0=a0		/* Store it in DLT.			*/

/****************************************************************/
/*								*/
/*			Block 4L				*/
/*								*/
/****************************************************************/

b4bl:	r1=DLT1		/**		UPZERO			**/
	r2=BL1		/* Set the initial point to BLi		*/
	i=128		/* i=128				*/
	a0=*r0		/* a0 = DLT				*/
	a1=i
	a0=a0		/* Check DLT = 0 ?			*/
	if eq a1=a1<<16 /* Yes, set WD1 = 0			*/
	k=a1
	y=*r1++
	do 6 {
	a1=k
	a0=*r0--	/* a0 = DLT				*/
	a0=a0^y  x=*r2	/* Check SG0 = SGi ?			*/
	if mi a1=-a1	/* No, set WD2 = - WD1			*/
	y=*r0++		/* y = 32640				*/
	p=x*y		/* WD3 = BLi * 32640			*/
	a0=p
	a0=a0<<1
	y=a0
	a1=a1+y  y=*r1++	/* BPLi = WD2 + WD3		*/
	*r2++=a1
	}
	y=*r0
	r1=DLT1		/* Update DLTi parameters.		*/
	do 6 {
	*r1zp:y
	}
	y=*r0		/**		PARREC			**/
	r1=SZL		/* y = DLT				*/
	a0=*r1++	/* a0 = SZL				*/
	a0=a0+y		/* a0 = PLT = DLT + SZL			*/
	r2=PLT
	*r2=a0		/* Store it in PLT location		*/
	a0=*r1		/**		RECONS			**/
	a0=a0+y		/* a0 = RLT = SL + DLT			*/
	r1=RLT1
	y=*r1++		/* Store it and update the RLTi		*/
	*r1--=y		/* parameters.				*/
	*r1=a0
	r2=AL1		/**		UPPOL2			**/
	y=*r2++		/* y = AL1				*/
	a0=y		/* a0 = AL1				*/
	a0=a0+y		/* a0 = WD1 = AL1 + AL1			*/
	y=a0		/* y = WD1 = AL1 + AL1			*/
	a0=y		/* a0 = WD1 = AL1 + AL1			*/
	a0=a0+y		/* a0 = WD1 = WD1 + WD1			*/
	y=a0		/* y = WD1 = WD1 + WD1			*/
	a0=y
	r1=PLT
	y=*r1++		/* y = PLT				*/
	a1=*r1++	/* a1 = PLT1				*/
	a1=a1^y		/* Check SG0 = SG1 ?			*/
	if pl a0=-a0	/* Yes, set WD2 = - WD1			*/
	x=a0		/* x = (0 - WD1) or (WD1)		*/
	a0=x
	a0=a0>>8
	a0=a0<<1	/* a0 = WD2 = WD2 >> 7			*/
	a1=*r1--	/* a1 = PLT2				*/
	a1=a1^y		/* Check SG0 = SG2 ?			*/
	a1=i		/* a1 = 128				*/
	if mi a1=-a1	/* No, set a1 = - 128			*/
	y=a0
	a1=a1+y  y=*r2	/* r2=AL2				*/
	x=32512
	p=x*y		/* p = AL2 * 32512			*/
	a0=p
	a0=a0<<1
	y=a0
	a1=a1+y		/* a1 = APL2 = WD4 + WD5		*/
	a0=a1
	if mi a1=-a1	/* a1 = | APL2 |			*/
	y=12288
	a1-y		/* | APL2 | > 12288 ?			*/
	if gt a1=y	/* Yes, set | APL2 | = 12288		*/
	a0=a0
	if mi a1=-a1	/* a1 = APL2				*/
	*r2--=a1	/* r2=AL2				*/
	y=192		/**		UPPOL1			**/
	a0=y		/* a0 = 192				*/
	y=*r1--		/* r1=PLT1				*/
	a1=*r1++	/* r1=PLT				*/
	*r1++=a1
	*r1=y
	a1=a1^y  y=*r2++	/* Check SG0 = SG1 ? r2=AL1	*/
	if mi a0=-a0	/* No, set a0 = - 192			*/
	x=32640
	p=x*y		/* p = WD2 = AL1 * 32640		*/
	a1=p
	a1=a1<<1
	y=a1
	a0=a0+y		/* a0 = APL1 = WD1 + WD2		*/
	y=15360		/* Set y = 15360			*/
	a1=y  y=*r2--	/* r2=APL2, a1 = 15360			*/
	a1=a1-y		/* a1 = WD3				*/
	y=a1
	a1=a0
	if mi a0=-a0	/* a0 = | APL1 |			*/
	a0-y		/* | APL1 | > WD3 ?			*/
	if gt a0=y	/* Yes, set | APL1 | = WD3		*/
	a1=a1
	if mi a0=-a0	/* a0 = APL1				*/
	*r2=a0		/* r2=AL1				*/

/****************************************************************/
/*								*/
/*			Block 3L				*/
/*								*/
/****************************************************************/

b3l:	x=32512		/**		LOGSCL			**/
	r0=NBL
	y=*r0		/* y = NBL				*/
	p=x*y		/* p = WD = NBL * 32512			*/
	a0=p
	a0=a0<<1	/* a0 = WD = NBL * 32512		*/
	a1=j
	y=TWL		/* Point to the base of WL table.	*/
	a1=a1+y
	pt=a1		/* Point to the location of WL(IL4).	*/
	y=a0  x=*pt++	/* y = WD, x = WL(IL4)			*/
	a0=x
	a0=a0+y		/* a0 = NBPL = WD + WL(IL4)		*/
	if mi a0=a0<<16 /* If NBPL < 0, then set NBPL = 0	*/
	y=18432
	a0-y		/* Compare NBPL with 18432 ?		*/
	if gt a0=y	/* If NBPL > 18432, then set NBPL=18432 */
	a1=a0>>4	/**		SCALEL			**/
	a1=a1>>1
	a1=a1>>1	/* a1 = NBPL >> 6			*/
	y=0x1f		/* Set y = 31				*/
	a1=a1&y  *r0=a0	 /* r0=NBL				*/
	y=TILB		/* Point to the base of ILB table	*/
	a1=a1+y
	pt=a1
	a0=a0>>8
	a0=a0>>4
	a0=a0<<1	/* a0 = WD2 = NBPL >> 11		*/
	y=a0  x=*pt++	/* y = WD2, x = ILB(WD1)		*/
	a1=y
	y=8		/* Set y = 8				*/
	a1=a1-y		/* a1 = WD2 - 8				*/
	a0=x
	if mi goto lab5 /* The following process is to do	*/
	a1=-a1		/* ILB(WD1) >> (8 - WD2)		*/
	c1=a1
	do 7 {
	if c1lt a0=a0<<1
	}
	goto lab6
lab5:	c1=a1
	do 12 {
	if c1lt a0=a0>>1
	}
lab6:	y=a0
	a0=y		/* a0 = WD3				*/
	a0=a0<<1
	a0=a0<<1	/* a0 = WD3 << 2			*/
	r0=DETL
	*r0=a0		/* Store it in DETL location		*/

/****************************************************************/
/*								*/
/*			Block 4H				*/
/*								*/
/****************************************************************/

b4ah:	r1=DH6		/**		FILTEZ			**/
	r2=SZH		/* Set initial points			*/
	r0=BH6
	y=0
	a1=y  x=*r0--	/* Clear a1, x = BH6			*/
	y=*r1--		/* y = DH6				*/
	do 6 {
	a0=y  *r2=a1
	a0=a0+y		/* WDn = DHn + DHn			*/
	y=a0		/* y = WDn				*/
	p=x*y  a1=*r2	/* p = BHn * WDn			*/
	a0=p  x=*r0--
	a0=a0<<1	/* Rescale the result.			*/
	y=a0
	a1=a1+y  y=*r1--	/* Accumulate the result.	*/
	}
	*r2++=a1	/* a1 = SZH				*/
	k=0
	a1=k
	redo 2		/**		FILTEP			**/
	*r2=a1		/* a1 = SPH				*/
	a1=*r2--	/* a1 = SPH				*/
	y=*r2++		/* y = SZH				*/
	a1=a1+y		/**		PREDIC			**/
	*r2=a1		/* a1=SH				*/

/****************************************************************/
/*								*/
/*			Block 1H				*/
/*								*/
/****************************************************************/

b1h:	r0=XH		/**		SUBTRA			**/
	y=*r2		/* y = SH				*/
	a1=*r0		/* a1 = XH				*/
	a1=a1-y		/* a1 = EH				*/
	r1=temp
	*r1=a1
	a1=*r1
	y=0xffff
	a1=a1^y  y=*r1	/* a1 = 32767 - EH			*/
	if mi a1=y	/* a1 = WD				*/
	x=4512		/**		QUANTH			**/
	r0=DETH
	y=*r0		/* y = DETH				*/
	p=x*y
	a0=p
	a0=a0<<1	/* a0 = 4512 * DETH			*/
	y=a0
	x=1
	a0=x
	a1-y		/* Compare WD with 4512 * DETH ?	*/
	if pl a0h=a0h+1 /* If greater, set a0 = 2		*/
	x=a0
	y=PTIH		/* The following process is to use table*/
	a0=y  a1=*r1	/* looking up to find the IH value	*/
	y=NTIH
	a1=a1
	if mi a0=y
	a1=x
	y=a0
	a1=a1+y
	pt=a1
	y=*r1  x=*pt++	/* x = IH				*/
	r0=IH
	*r0--=x		/* Store it in IH location.		*/

/****************************************************************/
/*								*/
/*			Block 2H				*/
/*								*/
/****************************************************************/

b2h:	a0=x		/**		INVQAH			**/
	y=TIH2		/* Point to the base of IH2 table.	*/
	a0=a0+y
	pt=a0		/* Point to the location of IH2		*/
	y=*r0  x=*pt++	/* r0=DETH, x = IH2			*/
	a0=x
	a1=a0
	if mi a0=-a0
	j=a0		/* j = IH2				*/
	y=TQQ2		/* Point to the base of QQ2 table.	*/
	a0=a0+y
	pt=a0		/* Point to the location of QQ2(IH2)	*/
	y=*r0  x=*pt++	/* r0=DETH, x = QQ2(IH2) << 3		*/
	a0=x
	a1=a1		/* Check SIH = 0 ?			*/
	if mi a0=-a0	/* No, negative it			*/
	x=a0		/* x = WD2				*/
	p=x*y		/* p = DETH * WD2			*/
	a0=p
	a0=a0<<1	/* a0 = DH = DETH * WD2			*/
	r0=DH
	*r0=a0		/* r0 = DH				*/

/****************************************************************/
/*								*/
/*			Block 4H				*/
/*								*/
/****************************************************************/

b4bh:	r1=DH1		/**		UPZERO		**/
	r2=BH1		/* Set the initial point to BHi	*/
	a0=*r0		/* a0 = DH			*/
	a1=i		/* a1 = i = 128			*/
	a0=a0		/* Check DH = 0 ?		*/
	if eq a1=a1<<16 /* Yes, set WD1 = 0		*/
	k=a1
	y=*r1++
	do 6 {
	a0=*r0--	/* r0=DH			*/
	a1=k
	a0=a0^y  x=*r2	/* Check SG0 = SGi ?		*/
	if mi a1=-a1	/* No, set WD2 = - WD1		*/
	y=*r0++		/* y = 32640			*/
	p=x*y		/* WD3 = BHi * 32640		*/
	a0=p
	a0=a0<<1
	y=a0
	a1=a1+y  y=*r1++	/* BPHi = WD2 + WD3	*/
	*r2++=a1
	}
	y=*r0
	r1=DH1		/* Update DHi parameters.	*/
	do 6 {
	*r1zp:y
	}
	y=*r0		/**		PARREC		**/
	r2=SZH		/* y = DH			*/
	a0=*r2++	/* a0 = SZH			*/
	a0=a0+y		/* a0 = PH = DH + SZH		*/
	r1=PH
	*r1=a0		/* Store it in PH location	*/
	a0=*r2		/**		RECONS		**/
	a0=a0+y		/* a0 = RH = SH + DH		*/
	r2=RH1
	y=*r2++		/* Store it and update the RHi	*/
	*r2--=y		/* parameters			*/
	*r2=a0
	r2=AH1		/**		UPPOL2		**/
	y=*r2++		/* y = AH1			*/
	a0=y		/* a0 = AH1			*/
	a0=a0+y		/* a0 = WD1 = AH1 + AH1		*/
	y=a0		/* y = WD1 = AH1 + AH1		*/
	a0=y		/* a0 = WD1 = AH1 + AH1		*/
	a0=a0+y		/* a0 = WD1 = WD1 + WD1		*/
	y=a0		/* y = WD1 = WD1 + WD1		*/
	a0=y
	y=*r1++		/* y = PH			*/
	a1=*r1++	/* a1 = PH1			*/
	a1=a1^y		/* Check SG0 = SG1 ?		*/
	if pl a0=-a0	/* Yes, set WD2 = - WD1		*/
	x=a0		/* x = (0 - WD1) or (WD1)	*/
	a0=x
	a0=a0>>8
	a0=a0<<1	/* a0 = WD2 = WD2 >> 7		*/
	a1=*r1--	/* r1 = PH2			*/
	a1=a1^y		/* Check SG0 = SG2 ?		*/
	a1=i		/* a1 = 128			*/
	if mi a1=-a1	/* No, set a1 = -128		*/
	y=a0
	a1=a1+y  y=*r2	/* r2=AH2			*/
	x=32512
	p=x*y		/* p = AH2 * 32512		*/
	a0=p
	a0=a0<<1
	y=a0
	a1=a1+y		/* a1 = APH2 = WD4 + WD5	*/
	a0=a1
	if mi a1=-a1	/* a1 = | APH2 |		*/
	y=12288
	a1-y		/* | APH2 | > 12288 ?		*/
	if gt a1=y	/* Yes, set | APH2 | = 12288	*/
	a0=a0
	if mi a1=-a1	/* a1 = APH2			*/
	*r2--=a1	/* r2=AH2			*/
	y=192		/**		UPPOL1		**/
	a0=y		/* a0 = 192			*/
	y=*r1--		/* r1 = PH1			*/
	a1=*r1++	/* r1 = PH			*/
	*r1++=a1
	*r1=y
	a1=a1^y  y=*r2++  /* Check SG0 = SG1 ? r2=AH1	*/
	if mi a0=-a0	/* No, set a0 = -192		*/
	x=32640
	p=x*y		/* p = WD2 = AH1 * 32640	*/
	a1=p
	a1=a1<<1
	y=a1
	a0=a0+y		/* a0 = APH1 = WD1 + WD2	*/
	y=15360		/* Set y = 15360		*/
	a1=y  y=*r2--	/* r2=APH2, a1 = 15360		*/
	a1=a1-y		/* a1 = WD3			*/
	y=a1
	a1=a0
	if mi a0=-a0	/* a0 = | APH1 |		*/
	a0-y		/* | APH1 | > WD3 ?		*/
	if gt a0=y	/* Yes, set | APH1 | = WD3	*/
	a1=a1
	if mi a0=-a0	/* a0 = APH1			*/
	*r2=a0		/* r2 = AH1			*/

/****************************************************************/
/*								*/
/*			Block 3H				*/
/*								*/
/****************************************************************/

b3h:	y=32512		/**		LOGSCH		**/
	r2=NBH
	x=*r2		/* x = NBH			*/
	p=x*y		/* p = WD = NBH * 32512		*/
	a1=p
	a1=a1<<1	/* a1 = WD = NBH * 32512	*/
	y=TWH		/* Point to the base of WH	*/
	a0=j		/* j = IH2			*/
	a0=a0+y
	pt=a0		/* Point to the location of WH(IH2) */
	y=a1  x=*pt++	/* y = WD, x = WH(IH2)		*/
	a0=x
	a0=a0+y		/* a0 = NBPH = WD + WH(IH2)	*/
	if mi a0=a0<<16 /* If NBPH < 0, then set NBPH = 0 */
	y=22528
	a0-y		/* Compare NBPH with 22528 ?	*/
	if gt a0=y	/* If NBPH > 22528, then set NBPH=22528 */
	a1=a0>>4
	a1=a1>>1
	a1=a1>>1	/* a1 = NBPH >> 6		*/
	y=31		/* Set y = 31			*/
	a1=a1&y  *r2=a0	  /* r2 = NBH			*/
	y=TILB		/* Point to the base of ILB table */
	a1=a1+y
	pt=a1
	a0=a0>>8
	a0=a0<<1
	a0=a0>>4	/* a0 = WD2 = NBPH >> 11	*/
	y=a0  x=*pt++	/* y = WD2, x = ILB(WD1)	*/
	a0=y
	y=10		/* Set y = 10			*/
	a0=a0-y		/* a0 = WD2 - 10		*/
	a1=x
	if mi goto lab7 /* The following process is to do */
	a0=-a0		/* ILB(WD1) >> (10 - WD2)	  */
	c1=a0
	do 5 {
	if c1lt a1=a1<<1
	}
	goto lab8
lab7:	c1=a0
	do 12 {
	if c1lt a1=a1>>1
	}
lab8:	y=a1
	a1=y		/* a1 = WD3			*/
	a1=a1<<1
	a1=a1<<1	/* a1 = WD3 << 2		*/
	r1=DETH
	*r1=a1		/* r1 = DEPH			*/

/****************************************************************/
/*								*/
/*		Combination of IH and IL			*/
/*								*/
/****************************************************************/
	r0=IH
	a0=*r0++	/* a0 = IH			*/
	a0=a0<<4
	a0=a0<<1
	a0=a0<<1	/* a0 = IH << 6			*/
	y=*r0++		/* y = IL			*/
	a0=a0+y		/* a0 = (IH)(IL)		*/
	a0=a0<<8	/* a0 = (IH)(IL) << 8		*/
	r0=OUT
	*r0=a0		/* Store it in OUT location	*/

/****************************************************************/
/*								*/
/*			End of Program				*/
/*								*/
/****************************************************************/


PTIL:	int	0x0
	int	0x3d
	int	0x3c
	int	0x3b
	int	0x3a
	int	0x39
	int	0x38
	int	0x37
	int	0x36
	int	0x35
	int	0x34
	int	0x33
	int	0x32
	int	0x31
	int	0x30
	int	0x2f
	int	0x2e
	int	0x2d
	int	0x2c
	int	0x2b
	int	0x2a
	int	0x29
	int	0x28
	int	0x27
	int	0x26
	int	0x25
	int	0x24
	int	0x23
	int	0x22
	int	0x21
	int	0x20

NTIL:	int	0x0
	int	0x3f
	int	0x3e
	int	0x1f
	int	0x1e
	int	0x1d
	int	0x1c
	int	0x1b
	int	0x1a
	int	0x19
	int	0x18
	int	0x17
	int	0x16
	int	0x15
	int	0x14
	int	0x13
	int	0x12
	int	0x11
	int	0x10
	int	0xf
	int	0xe
	int	0xd
	int	0xc
	int	0xb
	int	0xa
	int	0x9
	int	0x8
	int	0x7
	int	0x6
	int	0x5
	int	0x4

PTIH:	int	0x0
	int	0x3
	int	0x2

NTIH:	int	0x0
	int	0x1
	int	0x0

TIL4:	int	0
	int	-7
	int	-6
	int	-5
	int	-4
	int	-3
	int	-2
	int	-1
	int	7
	int	6
	int	5
	int	4
	int	3
	int	2
	int	1
	int	0

TWL:	int	-60
	int	-30
	int	58
	int	172
	int	334
	int	538
	int	1198
	int	3042

TILB:	int	2048
	int	2093
	int	2139
	int	2186
	int	2233
	int	2282
	int	2332
	int	2383
	int	2435
	int	2489
	int	2543
	int	2599
	int	2656
	int	2714
	int	2774
	int	2834
	int	2896
	int	2960
	int	3025
	int	3091
	int	3158
	int	3228
	int	3298
	int	3371
	int	3444
	int	3520
	int	3597
	int	3676
	int	3756
	int	3838
	int	3922
	int	4008

TQQ4:	int	0
	int	150
	int	323
	int	530
	int	786
	int	1121
	int	1612
	int	2557

TIH2:	int	-2
	int	-1
	int	2
	int	1

TQQ2:	int	0
	int	1616
	int	7408

TWH:	int	0
	int	-214
	int	798

.ram
temp:	int
temp1:	int 32640
DLT:	int
RLT1:	int
RLT2:	int
DLT1:	int
DLT2:	int
DLT3:	int
DLT4:	int
DLT5:	int
DLT6:	int
AL1:	int
AL2:	int
BL1:	int
BL2:	int
BL3:	int
BL4:	int
BL5:	int
BL6:	int
SZL:	int
SL:	int
PLT:	int
PLT1:	int
PLT2:	int
DETL:	int
NBL:	int
NBH:	int
DETH:	int
IH:	int
IL:	int
temp2:	int 32640
DH:	int
RH1:	int
RH2:	int
DH1:	int
DH2:	int
DH3:	int
DH4:	int
DH5:	int
DH6:	int
AH1:	int
AH2:	int
BH1:	int
BH2:	int
BH3:	int
BH4:	int
BH5:	int
BH6:	int
SZH:	int
SH:	int
PH:	int
PH1:	int
PH2:	int
XL:	int
XH:	int
OUT:	int
MEL:	int
tem2:	int
.endram
